// 003 u_c_gadget
// 20 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- MUST NOT use Canvas.Rectangle etc or any c_control or c_control_list classes
// should use NONE of this here: u_c_control  // get access to the true canvas

(*$r+*)

unit u_c_gadget;
  interface
    uses Graphics
        , u_c_basic_object
        , u_c_gadget_implementation
        ;

    type // -- here are the gadget kinds we need

         // -- in the ancestor,
         // --  place ALL the functions that will ever be needed (not all will be used
         // --  by each descendent gadget)
         c_abstract_gadget=
             class(c_basic_object)
               m_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
               m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height: Integer;

               m_on_mouse_down, m_on_mouse_move, m_on_mouse_up: t_on_gadget_mouse_down;

               Constructor create_abstract_gadget(p_name: String;
                   p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
                   p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);

               // -- draw text, line, poly, rect ...

               function f_display_gadget: String; Virtual;

               procedure save_graphic_context;
               procedure restore_graphic_context;
               procedure set_pen_color(p_color: Integer);
               procedure set_pen_mode(p_pen_mode: tPenMode);
               procedure set_brush_style(p_brush_style: tBrushStyle);
               procedure set_brush_color(p_brush_color: Integer);
               procedure set_font_height(p_font_height: Integer);
               function f_get_font_height: Integer;

               function f_character_width(p_character: Char): Integer;
               function f_character_height(p_character: Char): Integer;

               procedure draw_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer);
               procedure draw_triangle(p_left, p_top, p_width, p_height: Integer);
               procedure draw_rectangle(p_left, p_top, p_width, p_height: Integer);
               procedure draw_text(p_x, p_y: Integer; p_text: String); Virtual;
               procedure shrink_gadget(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer); Virtual;

               procedure fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);
               procedure draw_bitmap(p_x, p_y: Integer;  p_c_bitmap: tBitmap);

               procedure draw_content; Virtual; Abstract;
             end; // c_abstract_gadget

         c_button_gadget= class(c_abstract_gadget)
                            Constructor create_button_gadget(p_name: String;
                                p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
                                p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
                          end; // c_dialog_gadget

         c_label_gadget=
             class(c_abstract_gadget)
               Constructor create_label_gadget(p_name: String;
                   p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
                   p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
               procedure set_caption(p_caption: String);
             end; // c_dialog_gadget

         c_scrollbar_gadget=
             class(c_abstract_gadget)
               Constructor create_scrollbar_gadget(p_name: String;
                   p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
                   p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
               procedure draw_content; Override;
               procedure shrink_gadget(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer); Override;
               procedure set_position(p_position: Integer);
               function f_get_position: Integer;
             end; // c_scrollbar_gadget

         c_draw_surface_gadget=
             Class(c_abstract_gadget)
               Constructor create_draw_surface_gadget(p_name: String;
                   p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
                   p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
               procedure draw_content; Override;
             end; // c_draw_surface_gadge

  implementation
    uses SysUtils, u_c_display
        , u_graphic
        ;

    // -- c_abstract_gadget

    Constructor c_abstract_gadget.create_abstract_gadget(p_name: String;
        p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
        p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
      begin
        Inherited create_basic_object(p_name);
        m_c_abstract_gadget_implementation_ref:= p_c_abstract_gadget_implementation_ref;

        m_gadget_left:= p_gadget_left; m_gadget_top:= p_gadget_top;
        m_gadget_width:= p_gadget_width; m_gadget_height:= p_gadget_height;
      end; // create_abstract_gadget

    function c_abstract_gadget.f_display_gadget: String;
      begin
        Result:= m_name+ f_display_wh_rectangle(m_gadget_left, m_gadget_top,
            m_gadget_width, m_gadget_height);
      end; // f_display_gadget

    procedure c_abstract_gadget.save_graphic_context;
      begin
        m_c_abstract_gadget_implementation_ref.save_device_context;
      end; // save_graphic_context

    procedure c_abstract_gadget.restore_graphic_context;
      begin
        m_c_abstract_gadget_implementation_ref.restore_device_context;
      end; // restore_graphic_context

    procedure c_abstract_gadget.set_pen_color(p_color: Integer);
      begin
        m_c_abstract_gadget_implementation_ref.set_pen_color(p_color);
      end; // set_pen_color

    procedure c_abstract_gadget.set_pen_mode(p_pen_mode: tPenMode);
      begin
        m_c_abstract_gadget_implementation_ref.set_pen_mode(p_pen_mode);
      end; // set_pen_mode

    procedure c_abstract_gadget.set_brush_style(p_brush_style: tBrushStyle);
      begin
        m_c_abstract_gadget_implementation_ref.set_brush_style(p_brush_style);
      end; // set_brush_style

    procedure c_abstract_gadget.set_brush_color(p_brush_color: Integer);
      begin
        m_c_abstract_gadget_implementation_ref.set_brush_color(p_brush_color);
      end; // set_brush_color

    procedure c_abstract_gadget.set_font_height(p_font_height: Integer);
      begin
        if p_font_height< 0
          then display_bug_halt('neg_font');
        m_c_abstract_gadget_implementation_ref.canvas_set_font_height(p_font_height);
      end; // set_font_height

    function c_abstract_gadget.f_get_font_height: Integer;
      begin
        Result:= m_c_abstract_gadget_implementation_ref.f_canvas_get_font_height;
        if Result< 0
          then display_bug_halt('f_neg_font');
      end; // f_get_font_height

    function c_abstract_gadget.f_character_width(p_character: Char): Integer;
      begin
        Result:= m_c_abstract_gadget_implementation_ref.f_canvas_character_width(p_character);
      end; // f_character_width

    function c_abstract_gadget.f_character_height(p_character: Char): Integer;
      begin
        Result:= m_c_abstract_gadget_implementation_ref.f_canvas_character_height(p_character);
      end; // f_character_height

    procedure c_abstract_gadget.draw_text(p_x, p_y: Integer; p_text: String);
      begin
        with m_c_abstract_gadget_implementation_ref do
          device_text_out(p_x, p_y, p_text);
      end; // draw_text

    procedure c_abstract_gadget.draw_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer);
      begin
        with m_c_abstract_gadget_implementation_ref do
          canvas_line(p_x_1, p_y_1, p_x_2, p_y_2);
      end; // draw_line

    procedure c_abstract_gadget.draw_triangle(p_left, p_top, p_width, p_height: Integer);
        // -- use the device primitives to draw the shape
      begin
        with m_c_abstract_gadget_implementation_ref do
        begin
          display(f_display_wh_rectangle(p_left, p_top, 99, 99));
          canvas_line(p_left+ p_width div 2, p_top, p_left+ p_width, p_top);
          canvas_line(p_left+ p_width, p_top, p_left+ p_width, p_top+ p_height);
          canvas_line(p_left+ p_width, p_top+ p_height, p_left, p_top+ p_height);
          canvas_line(p_left, p_top+ p_height, p_left+ p_width div 2, p_top);
        end; // m_c_canvas
      end; // draw_triangle

    procedure c_abstract_gadget.draw_rectangle(p_left, p_top, p_width, p_height: Integer);
      begin
        m_c_abstract_gadget_implementation_ref.draw_rectangle(p_left, p_top, p_width, p_height);
      end; // draw_rectangle

    procedure c_abstract_gadget.shrink_gadget(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer);
      begin
        display('> shrink_gadget '+ f_display_wh_rectangle(m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height));

        Inc(m_gadget_left, p_delta_left);
        Inc(m_gadget_top, p_delta_top);
        Dec(m_gadget_width, p_delta_left+ p_delta_right);
        Dec(m_gadget_height, p_delta_top+ p_delta_bottom);

        display('< shrink_gadget '+ f_display_wh_rectangle(m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height));
      end; // shrink_gadget

    procedure c_abstract_gadget.fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);
      begin
        m_c_abstract_gadget_implementation_ref.fill_rectangle(p_left, p_top, p_width, p_height, p_color);
      end; // fill_rectangle

    procedure c_abstract_gadget.draw_bitmap(p_x, p_y: Integer; p_c_bitmap: tBitmap);
      begin
        m_c_abstract_gadget_implementation_ref.canvas_draw_bitmap(p_x, p_y, p_c_bitmap);
      end; // draw_bitmap

    // -- c_button_gadget

    Constructor c_button_gadget.create_button_gadget(p_name: String;
        p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
        p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
      begin
        Inherited create_abstract_gadget(p_name, p_c_abstract_gadget_implementation_ref,
            p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height);
      end; // create_button_gadget

    // -- c_label_gadget

    Constructor c_label_gadget.create_label_gadget(p_name: String;
        p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
        p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
      begin
        Inherited create_abstract_gadget(p_name, p_c_abstract_gadget_implementation_ref,
            p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height);
      end; // create_label_gadget

    procedure c_label_gadget.set_caption(p_caption: String);
      begin
        draw_text(m_gadget_left+ 4, m_gadget_top+ 2, p_caption);
      end; // c_dialog_gadget

    // -- c_draw_surface_gadget

    Constructor c_draw_surface_gadget.create_draw_surface_gadget(p_name: String;
        p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
        p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
      begin
        Inherited create_abstract_gadget(p_name, p_c_abstract_gadget_implementation_ref,
            p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height);

        draw_content;
      end; // create_draw_surface_gadget

    procedure c_draw_surface_gadget.draw_content;
      begin
        display('> draw_content '+ f_display_gadget);
        save_graphic_context;
        m_c_abstract_gadget_implementation_ref.set_pen_color(clFuchsia);
        // -- if wants to see the scrollbar control, must only use the ouline
        draw_rectangle(m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height);
        restore_graphic_context;
        display('< draw_content '+ m_name);
      end; // draw_content

    // -- c_scrollbar_gadget

    Constructor c_scrollbar_gadget.create_scrollbar_gadget(p_name: String;
        p_c_abstract_gadget_implementation_ref: c_abstract_gadget_implementation;
        p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height: Integer);
      begin
        Inherited create_abstract_gadget(p_name, p_c_abstract_gadget_implementation_ref,
            p_gadget_left, p_gadget_top, p_gadget_width, p_gadget_height);
      end; // create_scrollbar_gadget

    procedure c_scrollbar_gadget.draw_content;
      begin
        display('> draw_content '+ f_display_gadget);
        save_graphic_context;
        m_c_abstract_gadget_implementation_ref.set_pen_color(clBlue);
        draw_rectangle(m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height);
        restore_graphic_context;
        display('< draw_content '+ m_name);
      end; // draw_content

    procedure c_scrollbar_gadget.shrink_gadget(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom: Integer);
        // -- the right scroll-bar is some kind of "fat right edge"
        // --   => if moves the right edge, move the scroll bar
      begin
        display('> shrink_scroll_gadget '+ f_display_wh_rectangle(m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height)
          + ' '+ f_display_wh_rectangle(p_delta_left, p_delta_top, p_delta_right, p_delta_bottom));

        Dec(m_gadget_left, - p_delta_right);

        Inc(m_gadget_top, p_delta_top);
        Dec(m_gadget_height, (p_delta_top- p_delta_bottom) );

        display('< shrink_scroll_gadget '+ f_display_wh_rectangle(m_gadget_left, m_gadget_top, m_gadget_width, m_gadget_height));
      end; // shrink_gadget

    procedure c_scrollbar_gadget.set_position(p_position: Integer);
      begin
        m_c_abstract_gadget_implementation_ref.set_scrollbar_position(m_name, p_position);
      end; // set_position

    function c_scrollbar_gadget.f_get_position: Integer;
      begin
      end; // f_get_position

    begin // u_c_abstract_gadget
    end. // u_c_abstract_gadget
gadget
