// 003 u_c_gadget_implementation
// 19 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)

unit u_c_gadget_implementation;
  interface
    uses Graphics
        , u_c_basic_object
        , u_c_control  // get access to the true canvas
        ;

    type t_on_gadget_mouse_down= u_c_control.t_on_mouse_down;

         // -- here we encapsulate the platform specific code
         // --  - setting checks, reading scrollbar positions ...
         // --  - drawing line, character, fonts, colors
         // --    (some kind of "generic cross platform canvas")
         c_abstract_gadget_implementation=
             Class(c_basic_object)
               m_c_control_list_ref: c_control_list;

               constructor create_abstract_gadget_implementation(p_name: String;
                   p_c_control_list_ref: c_control_list);

               // -- label
               procedure set_label_caption(p_caption: String); Virtual; Abstract;

               // -- scrollbar
               procedure set_scrollbar_position(p_scrollbar_name: String; p_position: Integer); Virtual; Abstract;
               function f_get_scrollbar_position: Integer; Virtual; Abstract;

               // -- drawing
               procedure save_device_context; Virtual; Abstract;
               procedure restore_device_context; Virtual; Abstract;

               procedure set_pen_color(p_pen_color: Integer); Virtual; Abstract;
               procedure set_brush_style(p_brush_style: tBrushStyle); Virtual; Abstract;
               // -- here placed the code at this level rather than duplicating
               // --   it for the demo
               procedure set_pen_mode(p_pen_mode: tPenMode); Virtual;
               procedure set_brush_color(p_brush_color: Integer); Virtual;
               procedure canvas_set_font_height(p_font_height: Integer); Virtual;
               function f_canvas_get_font_height: Integer; Virtual;

               procedure canvas_draw_bitmap(p_x, p_y: Integer; p_c_bitmap: tBitMap); Virtual;

               function f_canvas_character_width(p_character: Char): Integer; Virtual; Abstract;
               function f_canvas_character_height(p_character: Char): Integer; Virtual; Abstract;

               procedure canvas_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer); Virtual; Abstract;
               procedure draw_rectangle(p_left, p_top, p_width, p_height: Integer); Virtual; Abstract;
               procedure device_text_out(p_x, p_y: Integer; p_text: String); Virtual; Abstract;

               procedure fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);  Virtual; Abstract;
             end; // c_abstract_gadget_implementation

         c_square_gadget_implementation=
             Class(c_abstract_gadget_implementation)
               constructor create_square_gadget_implementation(p_name: String;
                   p_c_control_list_ref: c_control_list);

               procedure set_scrollbar_position(p_scrollbar_name: String; p_position: Integer); Override;
               function f_get_scrollbar_position: Integer; Override;

               procedure save_device_context; Override;
               procedure restore_device_context; Override;
               procedure set_pen_color(p_pen_color: Integer); Override;
               procedure set_brush_style(p_brush_style: tBrushStyle); Override;

               function f_canvas_character_width(p_character: Char): Integer; Override;
               function f_canvas_character_height(p_character: Char): Integer; Override;

               procedure canvas_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer); Override;
               procedure draw_rectangle(p_left, p_top, p_width, p_height: Integer); Override;
               procedure device_text_out(p_x, p_y: Integer; p_text: String); Override;

               procedure fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);  Override;
             end; // c_square_gadget_implementation

         c_round_gadget_implementation=
             Class(c_abstract_gadget_implementation)
               constructor create_round_gadget_implementation(p_name: String;
                   p_c_control_list_ref: c_control_list);

               procedure set_scrollbar_position(p_scrollbar_name: String; p_position: Integer); Override;
               function f_get_scrollbar_position: Integer; Override;

               procedure save_device_context; Override;
               procedure restore_device_context; Override;
               procedure set_pen_color(p_pen_color: Integer); Override;
               procedure set_brush_style(p_brush_style: tBrushStyle); Override;

               function f_canvas_character_width(p_character: Char): Integer; Override;
               function f_canvas_character_height(p_character: Char): Integer; Override;

               procedure canvas_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer); Override;
               procedure draw_rectangle(p_left, p_top, p_width, p_height: Integer); Override;
               procedure device_text_out(p_x, p_y: Integer; p_text: String); Override;

               procedure fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);  Override;
             end; // c_abstract_gadget_implementation

  implementation
    uses SysUtils, Types, Forms
        , u_c_display
        , u_graphic
        , u_c_square_controls
        , u_c_round_controls
        ;

    // -- c_abstract_gadget_implementation

    constructor c_abstract_gadget_implementation.create_abstract_gadget_implementation(p_name: String;
        p_c_control_list_ref: c_control_list);
      begin
        Inherited create_basic_object(p_name);
        m_c_control_list_ref:= p_c_control_list_ref;
      end; // create_abstract_gadget_implementation

    procedure c_abstract_gadget_implementation.set_pen_mode(p_pen_mode: tPenMode);
      begin
        m_c_control_list_ref.m_c_canvas_ref.Pen.Mode:= p_pen_mode;
      end; // set_pen_mode

    procedure c_abstract_gadget_implementation.set_brush_color(p_brush_color: Integer);
      begin
        m_c_control_list_ref.m_c_canvas_ref.Brush.Color:= p_brush_color;
      end; // set_brush_color

    procedure c_abstract_gadget_implementation.canvas_set_font_height(p_font_height: Integer);
        // -- Height is the pixel area. It is negative, but we do not want
        // --  any of those negative values
      begin
        if p_font_height< 0
          then display_bug_halt('canv_neg_font ');
        m_c_control_list_ref.m_c_canvas_ref.Font.Height:= - p_font_height;
      end; // canvas_set_font_height

    function c_abstract_gadget_implementation.f_canvas_get_font_height: Integer;
      begin
        Result:= - m_c_control_list_ref.m_c_canvas_ref.Font.Height;
        if Result< 0
          then display_bug_halt('f_canv_neg_font ');
      end; // f_canvas_get_font_height

    procedure c_abstract_gadget_implementation.canvas_draw_bitmap(p_x, p_y: Integer; p_c_bitmap: tBitMap);
      begin
        m_c_control_list_ref.m_c_canvas_ref.Draw(p_x, p_y, p_c_bitmap);
      end; // canvas_draw_bitmap

    // -- c_square_gadget_implementation

    constructor c_square_gadget_implementation.create_square_gadget_implementation(p_name: String;
        p_c_control_list_ref: c_control_list);
      begin
        Inherited create_abstract_gadget_implementation(p_name, p_c_control_list_ref);
      end; // create_square_gadget_implementation

    procedure c_square_gadget_implementation.set_scrollbar_position(p_scrollbar_name: String; p_position: Integer);
      var l_square_scrollbar_control_index: Integer;
          l_c_square_scrollbar_control: c_square_scrollbar_control;
      begin
        l_square_scrollbar_control_index:= m_c_control_list_ref.f_index_of(p_scrollbar_name);
        if l_square_scrollbar_control_index<> -1
          then begin
              with m_c_control_list_ref.f_c_abstract_control(l_square_scrollbar_control_index) as c_square_scrollbar_control do
              begin
                m_position:= p_position;
                draw_control;
              end;
            end;
      end; // set_scrollbar_position

    function c_square_gadget_implementation.f_get_scrollbar_position: Integer;
      begin
      end; // f_get_scrollbar_position

    procedure c_square_gadget_implementation.save_device_context;
      begin
        m_c_control_list_ref.m_c_save_canvas_stack.push_save_canvas;
      end; // save_device_context

    procedure c_square_gadget_implementation.restore_device_context;
      begin
        m_c_control_list_ref.m_c_save_canvas_stack.pop_save_canvas;
      end; // restore_device_context

    procedure c_square_gadget_implementation.set_pen_color(p_pen_color: Integer);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          Pen.Color:= p_pen_color;
      end; // set_pen_color

    procedure c_square_gadget_implementation.set_brush_style(p_brush_style: tBrushStyle);
      begin
        m_c_control_list_ref.m_c_canvas_ref.Brush.Style:= p_brush_style;
      end; // set_brush_style

    function c_square_gadget_implementation.f_canvas_character_width(p_character: Char): Integer;
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          Result:= TextWidth(p_character);
      end; // f_canvas_character_width

    function c_square_gadget_implementation.f_canvas_character_height(p_character: Char): Integer;
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          Result:= TextHeight(p_character);
      end; // f_canvas_character_height

    procedure c_square_gadget_implementation.canvas_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
        begin
          MoveTo(p_x_1, p_y_1);
          LineTo(p_x_2, p_y_2);
        end;
      end; // canvas_line

    procedure c_square_gadget_implementation.draw_rectangle(p_left, p_top, p_width, p_height: Integer);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          Rectangle(p_left, p_top, p_left+ p_width, p_top+ p_height);
      end; // draw_rectangle

    procedure c_square_gadget_implementation.device_text_out(p_x, p_y: Integer; p_text: String);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          TextOut(p_x, p_y, p_text);
      end; // device_text_out

    procedure c_square_gadget_implementation.fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);
      var l_color: Integer;
      begin
        with m_c_control_list_ref, m_c_canvas_ref do
        begin
          m_c_save_canvas_stack.push_save_canvas;

          with m_c_save_canvas_stack do
            display('fill '+ f_display_wh_rectangle(p_left, p_top, p_width, p_height)
              + f_c_save_canvas(f_save_canvas_count- 1).f_display_save_canvas);
          l_color:= k_color_array[s_color_index mod 3];
          Inc(s_color_index);
          Pen.Color:= l_color;
          Brush.Color:= l_color;

          Brush.Style:= bsSolid;
          FillRect(Rect(p_left, p_top, p_left+ p_width, p_top+ p_height));
          m_c_save_canvas_stack.pop_save_canvas;
        end;
      end; // fill_rectangle

    // -- c_round_gadget_implementation

    constructor c_round_gadget_implementation.create_round_gadget_implementation(p_name: String;
        p_c_control_list_ref: c_control_list);
      begin
        Inherited create_abstract_gadget_implementation(p_name, p_c_control_list_ref);
      end; // create_round_gadget_implementation

    procedure c_round_gadget_implementation.set_scrollbar_position(p_scrollbar_name: String; p_position: Integer);
      var l_round_scrollbar_control_index: Integer;
          l_c_round_scrollbar_control: c_round_scrollbar_control;
      begin
        l_round_scrollbar_control_index:= m_c_control_list_ref.f_index_of(p_scrollbar_name);
        if l_round_scrollbar_control_index<> -1
          then begin
              with m_c_control_list_ref.f_c_abstract_control(l_round_scrollbar_control_index) as c_round_scrollbar_control do
              begin
                m_position:= p_position;
                draw_control;
              end;
            end;
      end; // set_scrollbar_position

    function c_round_gadget_implementation.f_get_scrollbar_position: Integer;
      begin
      end; // f_get_scrollbar_position

    procedure c_round_gadget_implementation.save_device_context;
      begin
        m_c_control_list_ref.m_c_save_canvas_stack.push_save_canvas;
      end; // save_device_context

    procedure c_round_gadget_implementation.restore_device_context;
      begin
        m_c_control_list_ref.m_c_save_canvas_stack.pop_save_canvas;
      end; // restore_device_context

    procedure c_round_gadget_implementation.set_pen_color(p_pen_color: Integer);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          Pen.Color:= p_pen_color;
      end; // set_pen_color

    procedure c_round_gadget_implementation.set_brush_style(p_brush_style: tBrushStyle);
      begin
        m_c_control_list_ref.m_c_canvas_ref.Brush.Style:= p_brush_style;
      end; // set_brush_style

    function c_round_gadget_implementation.f_canvas_character_width(p_character: Char): Integer;
      begin
        // -- +2: bogus, for the "rounding"
        with m_c_control_list_ref.m_c_canvas_ref do
          Result:= TextWidth(p_character)+ 2;
      end; // f_canvas_character_width

    function c_round_gadget_implementation.f_canvas_character_height(p_character: Char): Integer;
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          Result:= TextHeight(p_character)+ 2;
      end; // f_canvas_character_height

    procedure c_round_gadget_implementation.canvas_line(p_x_1, p_y_1, p_x_2, p_y_2: Integer);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
        begin
          MoveTo(p_x_1, p_y_1);
          LineTo(p_x_2, p_y_2);
        end;
      end; // canvas_line

    procedure c_round_gadget_implementation.draw_rectangle(p_left, p_top, p_width, p_height: Integer);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          RoundRect(p_left, p_top, p_left+ p_width, p_top+ p_height,
              10, 10) ; // k_button_rounding, k_button_rounding);
      end; // draw_rectangle

    procedure c_round_gadget_implementation.device_text_out(p_x, p_y: Integer; p_text: String);
      begin
        with m_c_control_list_ref.m_c_canvas_ref do
          TextOut(p_x, p_y, p_text);
      end; // device_text_out

    procedure c_round_gadget_implementation.fill_rectangle(p_left, p_top, p_width, p_height, p_color: Integer);
      begin
        with m_c_control_list_ref, m_c_canvas_ref do
        begin
          m_c_save_canvas_stack.push_save_canvas;
          Brush.Color:= p_color;
          Brush.Style:= bsSolid;
          Pen.Color:= p_color;
          FillRect(Rect(p_left, p_top, p_left+ p_width, p_top+ p_height));
          m_c_save_canvas_stack.pop_save_canvas;
        end;
      end; // fill_rectangle

    begin // u_c_abstract_widget
    end. // u_c_abstract_widget
