// 003 u_factory_and_bridge_patterns
// 20 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

(*$r+*)
unit u_factory_and_bridge_patterns;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, Buttons;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    xy_panel_: TPanel;
    Memo1: TMemo;
    ScrollBox1: TScrollBox;
    container_panel_: TPanel;
    PaintBox1: TPaintBox;
    Panel3: TPanel;
    exit_: TButton;
    clear_lexi_: TButton;
    clear_: TButton;
    create_square_world_: TButton;
    create_round_world_: TButton;
    GroupBox1: TGroupBox;
    square_: TRadioButton;
    round_: TRadioButton;
    create_widgets_: TButton;
    list_controls_: TButton;
    draw_control_: TButton;
    create_mixed_with_factory_: TButton;
    create_bridge_: TButton;
    use_: TButton;
    label_: TCheckBox;
    scroll_: TCheckBox;
    button_: TCheckBox;
    surface_: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure exit_Click(Sender: TObject);
    procedure clear_Click(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure clear_lexi_Click(Sender: TObject);
    procedure create_widgets_Click(Sender: TObject);
    procedure list_controls_Click(Sender: TObject);
    procedure draw_control_Click(Sender: TObject);
    procedure create_square_world_Click(Sender: TObject);
    procedure create_round_world_Click(Sender: TObject);
    procedure create_mixed_with_factory_Click(Sender: TObject);
    procedure create_bridge_Click(Sender: TObject);
    procedure use_Click(Sender: TObject);
  private
  public
    procedure increase_the_font_height_click;
    procedure decrease_the_font_height_click;
    procedure on_insert_button_click;
  end;

    var Form1: TForm1;

  implementation
    uses u_types_constants, u_c_log, u_c_display
        , u_dir, u_file
        , u_graphic

        , u_c_control
        , u_c_square_controls
        , u_c_round_controls

        , u_c_widget_factory

        , u_c_gadget_implementation
        , u_c_gadget

        , u_c_gadget_factory

    ;


    {$R *.DFM}

    var g_c_control_list: c_control_list= Nil;

    procedure display_square_or_round;
      begin
        if Form1.square_.Checked
          then display('width-height square')
          else display('p1-p2 rounded');
      end; // display_square_or_round

    // -- mouse events propagated to the control list

    procedure TForm1.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
          Shift: TShiftState; X, Y: Integer);
      begin
        if g_c_control_list<> Nil
          then g_c_control_list.handle_mouse_down(X, Y);
      end; // PaintBox1MouseDown

    procedure TForm1.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
        Y: Integer);
      begin
        xy_panel_.Caption:= Format('x =%4d, y =%4d', [X, Y]);
        if g_c_control_list<> Nil
          then g_c_control_list.handle_mouse_move(X, Y);
      end; // PaintBox1MouseMove

    procedure TForm1.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
          Shift: TShiftState; X, Y: Integer);
      begin
        if g_c_control_list<> Nil
          then g_c_control_list.handle_mouse_up(X, Y);
      end; // PaintBox1MouseUp

    // -- events

    procedure TForm1.FormCreate(Sender: TObject);
      begin
        initialize_display(Memo1.Lines);
        initialize_default_log;

        with container_panel_ do
        begin
          Left:= 2; Top:= 2; Width:= ScrollBox1.Width- 35; Height:= ScrollBox1.Height- 20;
        end;

        with PaintBox1 do
        begin
          Left:= 2; Top:= 2; Width:= container_panel_.Width- 4; Height:= container_panel_.Height- 4;
        end;

        // justification_edit_.Text:= IntToStr(k_justification_width);
      end; // FormCreate

    procedure TForm1.exit_Click(Sender: TObject);
      begin
        Close;
      end; // close_Click

    procedure TForm1.clear_Click(Sender: TObject);
      begin
        clear_display;
      end; // clear_Click

    procedure TForm1.clear_lexi_Click(Sender: TObject);
        // -- remove any drawing in the drawing area
      begin
        PaintBox1.Repaint;
      end; // clear_lexi_Click

    // -- user action on tForm

    procedure TForm1.draw_control_Click(Sender: TObject);
      begin
        if g_c_control_list<> Nil
          then g_c_control_list.draw_control_list;
      end; // draw_control_Click

    procedure TForm1.list_controls_Click(Sender: TObject);
      begin
        if g_c_control_list<> Nil
          then g_c_control_list.display_control_list;
      end; // list_Click

    procedure TForm1.on_insert_button_click;
        // -- this is a "click event"
      begin
        display('insert');
        Panel1.Color:= ClRed;
      end; // on_insert_button_click

    procedure TForm1.create_square_world_Click(Sender: TObject);
      var l_c_square_button: c_square_button_control;
      begin
        g_c_control_list:= c_control_list.create_control_list('surface', PaintBox1);

        l_c_square_button:= c_square_button_control.create_square_button_control('Insert',
            g_c_control_list, 10, 10, 120, 35, on_insert_button_click);
      end; // create_square_world_Click

    procedure TForm1.create_round_world_Click(Sender: TObject);
      var l_c_round_button: c_round_button_control;
      begin
        g_c_control_list:= c_control_list.create_control_list('surface', PaintBox1);

        l_c_round_button:= c_round_button_control.create_round_button_control('Insert',
            g_c_control_list, 10, 40, 110, 0.2272, on_insert_button_click);
      end; // create_round_world_Click

    procedure TForm1.create_mixed_with_factory_Click(Sender: TObject);
        // -- use an abstract factory to build the "button lexi" setup
      var l_c_widget_factory: c_abstract_widget_factory;
          l_running_y, l_running_x: Integer;

      procedure add_button(p_caption: String; p_width: Integer; p_click_event: t_on_click);
        begin
          with l_c_widget_factory do
            f_c_create_button_widget(p_caption, l_running_x, l_running_y, p_width, k_button_default_height,
                p_click_event);
          Inc(l_running_x, p_width+ 5);
        end; // add_button

      begin // create_Click
        g_c_control_list.Free;
        g_c_control_list:= c_control_list.create_control_list('surface', PaintBox1);

        l_c_widget_factory:= c_square_widget_factory.create_square_widget_factory('square_fact', g_c_control_list);
        // -- now all the calls look the same, whether round or square library
        l_running_x:= 5;
        l_running_y:= 5;

        add_button('<justify_button', 43, Nil);
        add_button('|justify|_button', 43, Nil);
        add_button('justify >_button', 47, Nil);
        add_button('justify Z_button', 47, Nil);

        l_c_widget_factory:= c_round_widget_factory.create_round_widget_factory('round_fact', g_c_control_list);
        l_running_x:= 5;
        l_running_y:= 35;

        add_button('<justify_button', 43, Nil);
        add_button('|justify|_button', 43, Nil);
        add_button('justify >_button', 47, Nil);
        add_button('justify Z_button', 47, Nil);
      end; // create_Click

    // -- the widgets

    var g_c_font_label_widget: c_abstract_label_widget= Nil;
        g_font_height: Integer= 18;

    procedure display_font_height;
      begin
        display(IntToStr(g_font_height));
        with Form1 do
         if square_.Checked
           then
             with (g_c_font_label_widget as c_square_label_widget).m_c_square_label_control do
             begin
               m_caption:= 'font='+ IntToStr(g_font_height);
               draw_control;
             end
           else
             with (g_c_font_label_widget as c_round_label_widget).m_c_round_label_control do
             begin
               m_caption:= 'font='+ IntToStr(g_font_height);
               draw_control;
             end
      end; // display_font_height

    procedure TForm1.increase_the_font_height_click;
      begin
        Inc(g_font_height, 5);
        display_font_height;
      end; // increase_the_font_height_click

    procedure TForm1.decrease_the_font_height_click;
      begin
        Dec(g_font_height, 5);
        display_font_height;
      end; // decrease_the_font_height_click

    procedure TForm1.create_widgets_Click(Sender: TObject);
        // -- use an abstract factory to build the "button lexi" setup
      var l_c_widget_factory: c_abstract_widget_factory;
          l_running_y, l_running_x: Integer;

      procedure add_button(p_caption: String; p_width: Integer; p_click_event: t_on_click);
        begin
          with l_c_widget_factory do
            f_c_create_button_widget(p_caption, l_running_x, l_running_y, p_width, k_button_default_height,
                p_click_event);
          Inc(l_running_x, p_width+ 5);
        end; // add_button

      function f_c_add_label_widget(p_caption: String; p_width: Integer): c_abstract_label_widget;
        begin
          with l_c_widget_factory do
            Result:= f_c_create_label_widget(p_caption, l_running_x, l_running_y, p_width, k_button_default_height);
          Inc(l_running_x, p_width+ 5);
        end; // f_c_add_label_widget

      var l_memo_rectangle: t_wh_rectangle;

      begin // create_Click
        g_c_control_list:= c_control_list.create_control_list('surface', PaintBox1);

        display_square_or_round;
        if square_.Checked
          then l_c_widget_factory:= c_square_widget_factory.create_square_widget_factory('square_fact', g_c_control_list)
          else l_c_widget_factory:= c_round_widget_factory.create_round_widget_factory('round_fact', g_c_control_list);

        // -- now all the calls look the same, whether round or square library
        with l_c_widget_factory do
        begin
          l_running_x:= 5;
          l_running_y:= 5;

          add_button('frame_button', 35, Nil);
          add_button('scroll_button', 35, Nil);

          l_memo_rectangle:= f_wh_rectangle(5, 5+ 4* (k_button_default_height+ 5),
              PaintBox1.Width- 10- 20, PaintBox1.Height- 5- 4* (k_button_default_height+ 5)- 5);
          with l_memo_rectangle do
          begin
            f_c_create_draw_surface_widget('memo', m_left, m_top, m_width, m_height,
                Nil, Nil, Nil);
            // -- add a custom scrollbar
            f_c_create_scrollbar_widget('scroll_bar', m_left+ m_width+ 5, m_top, 15, m_height);
          end;
        end; // with l_c_widget_factory

        Inc(l_running_x, 5);
        add_button('-_button', 15, decrease_the_font_height_click);
        g_c_font_label_widget:= f_c_add_label_widget('font='+ IntToStr(g_font_height), 45);
        add_button('+_button', 15, increase_the_font_height_click);
        Inc(l_running_x, 5);
        add_button('Esc_button', 25, Nil); // undo_command_click);

        Inc(l_running_x, 5);

        Inc(l_running_y, k_button_default_height+ 5);
        l_running_x:= 5;
        add_button('list_button', 25, Nil); // list_glyphs_click);
        add_button('count_button', 35, Nil); // count_glyphs_click);
        add_button('spell_button', 33, Nil); // check_spelling_click);

        Inc(l_running_y, k_button_default_height+ 5);
        l_running_x:= 5;
        add_button('<justify_button', 43, Nil); //justify_to_the_left_click);
        add_button('|justify|_button', 43, Nil); // justify_both_sides_click);
        add_button('justify >_button', 47, Nil); // justify_to_the_right_click);
        add_button('justify Z_button', 47, Nil); // justify_page_click);

        Inc(l_running_y, k_button_default_height+ 5);
        l_running_x:= 5;
        add_button('letter', 40, Nil); // insert_letter_click);
        add_button('rectangle', 53, Nil); // insert_rectangle_click);
        add_button('bitmap', 50, Nil); // insert_bitmap_click);
      end; // create_Click

    // -- the gadgets

    var g_c_button_gadget: c_button_gadget;
        g_c_label_gadget: c_label_gadget;
        g_c_scrollbar_gadget: c_scrollbar_gadget;
        g_c_draw_surface_gadget: c_draw_surface_gadget;

    procedure TForm1.create_bridge_Click(Sender: TObject);
        // -- use an abstract factory to build the "button lexi" setup
      var l_c_gadget_factory: c_abstract_gadget_factory;
          l_c_gadget_implementation: c_abstract_gadget_implementation;
          l_memo_rectangle: t_wh_rectangle;
      begin
        // -- 1 - the "windowing libraries"
        g_c_control_list.Free;
        g_c_control_list:= c_control_list.create_control_list('surface', PaintBox1);

        // -- 2 - the abstract factory
        display_square_or_round;
        if square_.Checked
          then l_c_gadget_factory:= c_square_gadget_factory.create_square_gadget_factory('square_fact', g_c_control_list)
          else l_c_gadget_factory:= c_round_gadget_factory.create_round_gadget_factory('round_fact', g_c_control_list);

        // -- now all the calls look the same, whether round or square library
        with l_c_gadget_factory do
        begin
          // -- 3 - the gadget implementation (a ref is saved inside the factory)
          l_c_gadget_implementation:= f_c_create_gadget_implementation('implem');
          if l_c_gadget_factory._m_c_abstract_gadget_implementation_ref= Nil
            then display_bug_halt('still_nil');

          // -- 4 some sample gadgets
          if button_.Checked
            then g_c_button_gadget:= f_c_create_button_gadget('Insert',
                5, 5, 150, k_button_default_height, Nil);
          if label_.Checked
            then g_c_label_gadget:= f_c_create_label_gadget('font size',
                5, 5+ k_button_default_height+ 5, 135, k_button_default_height);
          l_memo_rectangle:= f_wh_rectangle(5, 5+ 2* (k_button_default_height+ 5),
              PaintBox1.Width- 10- 25, PaintBox1.Height- 5- 2* (k_button_default_height+ 5)- 5);
          with l_memo_rectangle do
          begin
            if scroll_.Checked
              then g_c_scrollbar_gadget:= f_c_create_scrollbar_gadget('scroll',
                  m_left+ m_width+ 5, m_top, 20, m_height);
            if surface_.Checked
              then g_c_draw_surface_gadget:= f_c_create_draw_surface_gadget('memo',
                  m_left, m_top, m_width, m_height,
                  Nil, Nil, Nil);
          end; // with l_memo_rectangle
          Free;
        end; // with l_c_widget_factory
      end; // create_bridge_Click

    procedure TForm1.use_Click(Sender: TObject);
      var l_c_bitmap: tBitmap;
      begin
        if label_.Checked
          then g_c_label_gadget.set_caption('new caption');
        if scroll_.Checked
          then g_c_scrollbar_gadget.set_position(20);
        if surface_.Checked
          then
            with g_c_draw_surface_gadget do
            begin
              draw_text(10, 110, 'ok');
              set_pen_color(clRed);
              draw_rectangle(30, 110, 70, 20);
              l_c_bitmap:= tBitmap.Create;
              l_c_bitmap.LoadFromFile('client_pc.bmp');
              draw_bitmap(150, 110, l_c_bitmap);
            end;
      end; // handle_Click

end.


