// 002 u_c_round_controls
// 10 dec 2004

// -- (C) Felix John COLIBRI 2004
// -- documentation: http://www.felix-colibri.com

// -- P1-P2 round rectangle windowing system

(*$r+*)

unit u_c_round_controls;
  interface
    uses Classes
        , u_c_control
        ;

    const k_button_rounding= 10;
          k_round_control_default_percent= 0.45;

    type c_round_abstract_control= Class(c_abstract_control)
                                     m_x_1, m_y_1, m_x_2, m_y_2: Integer;

                                     constructor create_round_abstract_control(p_name: String;
                                        p_c_control_list_ref: c_control_list;
                                        p_x_1, p_y_1, p_size: Integer; p_percent: Double);

                                     function f_hit_test(p_x, p_y: Integer): Boolean; Override;
                                     procedure draw_blank_rectangle;

                                     procedure draw_round_rectangle(p_x_1, p_y_1, p_size, p_percent: Integer);
                                   end; // c_round_abstract_control

         c_round_button_control= Class(c_round_abstract_control)
                                   m_caption: String;
                                   constructor create_round_button_control(p_name: String;
                                       p_c_control_list_ref: c_control_list;
                                       p_x_1, p_y_1, p_size: Integer; p_percent: Double;
                                       p_on_click: t_on_click);
                                   procedure handle_control_mouse_down(p_x, p_y: Integer); Override;
                                   procedure handle_control_mouse_up(p_x, p_y: Integer); Override;
                                   procedure draw_control; Override;
                                 end; // c_round_button_control

         c_round_label_control= Class(c_round_abstract_control)
                                   m_caption: String;
                                   constructor create_round_label_control(p_name: String;
                                       p_c_control_list_ref: c_control_list;
                                       p_x_1, p_y_1, p_size: Integer; p_percent: Double);
                                   procedure draw_control; Override;
                                 end; // c_round_label_control

         c_round_draw_surface_control= Class(c_round_abstract_control)
                                         constructor create_round_draw_surface_control(p_name: String;
                                             p_c_control_list_ref: c_control_list;
                                             p_x_1, p_y_1, p_size: Integer; p_percent: Double;
                                             p_on_mouse_down, p_on_mouse_move, p_on_mouse_up: t_on_mouse_down);

                                         function f_c_self: c_round_draw_surface_control;
                                         procedure handle_control_mouse_down(p_x, p_y: Integer); Override;
                                         procedure handle_control_mouse_move(p_x, p_y: Integer); Override;
                                         procedure handle_control_mouse_up(p_x, p_y: Integer); Override;

                                         procedure draw_control; Override;
                                       end; // c_round_draw_surface_control

         c_round_scrollbar_control= Class(c_round_abstract_control)
                                      m_min, m_max, m_position: Integer;
                                      m_mouse_is_down: Boolean;

                                      constructor create_round_scrollbar_control(p_name: String;
                                          p_c_control_list_ref: c_control_list;
                                          p_x_1, p_y_1, p_size: Integer; p_percent: Double);
                                      function f_c_self: c_round_scrollbar_control;
                                      procedure handle_control_mouse_down(p_x, p_y: Integer); Override;
                                      procedure handle_control_mouse_move(p_x, p_y: Integer); Override;
                                      procedure handle_control_mouse_up(p_x, p_y: Integer); Override;

                                      procedure draw_cursor(p_position: Integer);
                                      procedure draw_control; Override;
                                    end; // c_round_scrollbar_control


  implementation
    uses SysUtils, Graphics
        , u_c_display, u_strings
        , u_c_pen_brush_stack
        ;

    // -- c_round_abstract_control

    constructor c_round_abstract_control.create_round_abstract_control(p_name: String;
        p_c_control_list_ref: c_control_list;
        p_x_1, p_y_1, p_size: Integer; p_percent: Double);
      begin
        Inherited create_abstract_control(p_name, p_c_control_list_ref);
        m_x_1:= p_x_1; m_y_1:= p_y_1;
        m_x_2:= p_x_1+ p_size;
        m_y_2:= m_y_1+ Round(p_size* p_percent);
      end; // create_round_abstract_control

    function c_round_abstract_control.f_hit_test(p_x, p_y: Integer): Boolean;
      begin
        Result:= (p_x>= m_x_1) and (p_x< m_x_2)
          and (p_y>= m_y_1) and (p_y< m_y_2)
      end; // handle_control_mouse_down

    procedure c_round_abstract_control.draw_blank_rectangle;
        // -- remove any edge
      begin
        m_c_control_list_ref.m_c_save_canvas_stack.push_save_canvas;

        with m_c_control_list_ref.m_c_canvas_ref do
        begin
          Pen.Color:= clWhite;
          Rectangle(m_x_1, m_y_1, m_x_2, m_y_2);
        end; // with m_c_control_list_ref.

        m_c_control_list_ref.m_c_save_canvas_stack.pop_save_canvas;
      end; // draw_blank_rectangle

    procedure c_round_abstract_control.draw_round_rectangle(p_x_1, p_y_1, p_size, p_percent: Integer);
      begin
        with m_c_control_list_ref, m_c_canvas_ref do
          RoundRect(m_x_1, m_y_1, m_x_2, m_y_2,
              k_button_rounding, k_button_rounding);
      end; // draw_round_rectangle

    // -- c_round_button_control

    constructor c_round_button_control.create_round_button_control(p_name: String;
        p_c_control_list_ref: c_control_list;
        p_x_1, p_y_1, p_size: Integer; p_percent: Double;
        p_on_click: t_on_click);
      begin
        Inherited create_round_abstract_control(p_name, p_c_control_list_ref,
            p_x_1, p_y_1, p_size, p_percent);
        m_caption:= f_remove_end_if_end_is_equal_to(m_name, '_button');

        m_on_click:= p_on_click;
        draw_control;
      end; // create_round_button_control

    procedure c_round_button_control.handle_control_mouse_down(p_x, p_y: Integer);

      procedure display_button_down;
        var l_caption_top: Integer;
        begin
          with m_c_control_list_ref, m_c_canvas_ref do
          begin
            m_c_control_list_ref.m_c_save_canvas_stack.push_save_canvas;

            Brush.Color:= clBlack;
            RoundRect(m_x_1, m_y_1,
                m_x_2- k_button_shadow, m_y_2- k_button_shadow,
                k_button_rounding, k_button_rounding);
            Brush.Color:= clWhite;
            RoundRect(m_x_1+ k_button_shadow, m_y_1+ k_button_shadow,
                m_x_2, m_y_2,
                k_button_rounding, k_button_rounding);

            l_caption_top:= m_y_1+ k_button_shadow
                + (m_y_2+ 1- m_y_1- TextHeight(Self.m_name)- k_button_shadow) div 2;
            TextOut(m_x_1+ 4, l_caption_top, Self.m_caption);

            m_c_control_list_ref.m_c_save_canvas_stack.pop_save_canvas;
          end;
        end; // display_button_down

      begin // handle_control_mouse_down
        // -- visual effect
        display_button_down;

        if Assigned(m_on_click)
          then m_on_click;
      end; // handle_control_mouse_down

    procedure c_round_button_control.handle_control_mouse_up(p_x, p_y: Integer);
      begin
        draw_control;
      end; // handle_control_mouse_up

    procedure c_round_button_control.draw_control;
      var l_caption_top: Integer;
      begin
        with m_c_control_list_ref, m_c_canvas_ref do
        begin
          draw_blank_rectangle;

          RoundRect(m_x_1+ k_button_shadow, m_y_1+ k_button_shadow,
              m_x_2, m_y_2, k_button_rounding, k_button_rounding);
          RoundRect(m_x_1, m_y_1,
              m_x_2- k_button_shadow, m_y_2- k_button_shadow,
              k_button_rounding, k_button_rounding);

          l_caption_top:= m_y_1+ (m_y_2+ 1- m_y_1- TextHeight(Self.m_name)- k_button_shadow) div 2;
          TextOut(m_x_1+ 4, l_caption_top, Self.m_caption);
        end; // with m_c_control_list_ref
      end; // draw_control

    // -- c_round_label_control

    constructor c_round_label_control.create_round_label_control(p_name: String;
        p_c_control_list_ref: c_control_list;
        p_x_1, p_y_1, p_size: Integer; p_percent: Double);
      begin
        Inherited create_round_abstract_control(p_name, p_c_control_list_ref,
            p_x_1, p_y_1, p_size, p_percent);
        m_caption:= f_remove_end_if_end_is_equal_to(m_name, '_label');

        draw_control;
      end; // create_round_label_control

    procedure c_round_label_control.draw_control;
      var l_caption_top: Integer;
      begin
        with m_c_control_list_ref, m_c_canvas_ref do
        begin
          RoundRect(m_x_1, m_y_1,
              m_x_2, m_y_2, k_button_rounding, k_button_rounding);

          l_caption_top:= m_y_1+ (m_y_2+ 1- m_y_1- TextHeight(Self.m_name)) div 2;
          TextOut(m_x_1+ 4, l_caption_top, Self.m_caption);
        end; // with m_c_control_list_ref
      end; // draw_control

    // -- c_round_draw_surface_control

    constructor c_round_draw_surface_control.create_round_draw_surface_control(p_name: String;
        p_c_control_list_ref: c_control_list;
        p_x_1, p_y_1, p_size: Integer; p_percent: Double;
        p_on_mouse_down, p_on_mouse_move, p_on_mouse_up: t_on_mouse_down);
      begin
        Inherited create_round_abstract_control(p_name, p_c_control_list_ref,
            p_x_1, p_y_1, p_size, p_percent);

        m_on_mouse_down:= p_on_mouse_down;
        m_on_mouse_move:= p_on_mouse_move;
        m_on_mouse_up:= p_on_mouse_up;

        draw_control;
      end; // create_round_draw_surface_control

    function c_round_draw_surface_control.f_c_self: c_round_draw_surface_control;
      begin
        Result:= Self;
      end; // f_c_self

    procedure c_round_draw_surface_control.handle_control_mouse_down(p_x, p_y: Integer);
      begin
        if Assigned(m_on_mouse_down)
          then m_on_mouse_down(p_x, p_y);
      end; // handle_control_mouse_down

    procedure c_round_draw_surface_control.handle_control_mouse_move(p_x, p_y: Integer);
      begin
        if Assigned(m_on_mouse_move)
          then m_on_mouse_move(p_x, p_y);
      end; // handle_control_mouse_move

    procedure c_round_draw_surface_control.handle_control_mouse_up(p_x, p_y: Integer);
      begin
        if Assigned(m_on_mouse_up)
          then m_on_mouse_up(p_x, p_y);
      end; // handle_control_mouse_up

    procedure c_round_draw_surface_control.draw_control;
      begin
        draw_blank_rectangle;
        with m_c_control_list_ref, m_c_canvas_ref do
          RoundRect(m_x_1, m_y_1,
              m_x_2, m_y_2, k_button_rounding, k_button_rounding);
      end; // draw_control

    // -- c_round_scrollbar_control

    constructor c_round_scrollbar_control.create_round_scrollbar_control(p_name: String;
        p_c_control_list_ref: c_control_list;
        p_x_1, p_y_1, p_size: Integer; p_percent: Double);
      begin
        Inherited create_round_abstract_control(p_name, p_c_control_list_ref,
            p_x_1, p_y_1, p_size, p_percent);

        m_max:= 100;
        draw_control;
      end; // create_round_scrollbar_control

    function c_round_scrollbar_control.f_c_self: c_round_scrollbar_control;
      begin
        Result:= Self;
      end; // f_c_self

    procedure c_round_scrollbar_control.handle_control_mouse_down(p_x, p_y: Integer);
      begin
        display('scroll_down');
        m_mouse_is_down:= True;
      end; // handle_control_mouse_down

    procedure c_round_scrollbar_control.handle_control_mouse_move(p_x, p_y: Integer);
      var l_position: Integer;
      begin
        if m_mouse_is_down
          then begin
              l_position:= m_min+ Round(1.0* (m_max- m_min)* (p_y- m_y_1)/ (m_y_2- m_y_1));
              draw_cursor(l_position);
            end;
      end; // handle_control_mouse_move

    procedure c_round_scrollbar_control.handle_control_mouse_up(p_x, p_y: Integer);
      begin
        m_mouse_is_down:= False;
      end; // handle_control_mouse_up

    procedure c_round_scrollbar_control.draw_cursor(p_position: Integer);
        // -- draw the cursor

      procedure do_draw(p_the_position: Integer);
        var l_cursor_y_1: Integer;
        begin
          with m_c_control_list_ref, m_c_canvas_ref do
          begin
            l_cursor_y_1:= m_y_1+ 2+ Round(1.0* (p_the_position / m_max)
                * (m_y_2- m_y_1- k_scrollbar_cursor_height- 2));
            RoundRect(m_x_1+ 2, l_cursor_y_1,
                m_x_2- 3, l_cursor_y_1+ k_scrollbar_cursor_height,
                k_button_rounding, k_button_rounding);
          end; // with m_c_control_list_ref
        end; // do_draw

      begin // draw_cursor
        if p_position<> m_position
          then begin
              m_c_control_list_ref.m_c_save_canvas_stack.push_save_canvas;
              with m_c_control_list_ref, m_c_canvas_ref do
                Pen.Color:= clWhite;
              do_draw(m_position);
              m_c_control_list_ref.m_c_save_canvas_stack.pop_save_canvas;
            end;

        m_position:= p_position;
        do_draw(m_position);
      end; // draw_cursor

    procedure c_round_scrollbar_control.draw_control;
      begin
        draw_blank_rectangle;
        // -- the outside
        with m_c_control_list_ref, m_c_canvas_ref do
          RoundRect(m_x_1, m_y_1, m_x_2, m_y_2, k_button_rounding, k_button_rounding);
        draw_cursor(m_position);
      end; // draw_control

    begin // u_c_yyy
    end. // u_c_yyy


